/*
// There are two kinds of info lines: score lines and nonscore lines.
// There are two kinds of score lines: simple score lines and total
// score lines.  See READ_ME for further details.
//
// The normal use is to append scores to a simple score line.
// However, the user is allowed to append to any info line.
//
// The user is interactively queried to identify the info lines
// to be appended to and the number of scores to be appended.
// Then this function repeatedly prints on the screen
//    name line
//    info line
// with the cursor sitting at the end of the info line.
// The user supplied data is read and the info line is remade.
// All lines from the input file are copied to the output file,
// except that the remade info line replaces the original.
//
//   cll_info_lines := create a linked list of info lines
//     cnt_ll_lines := count a linked list of lines
//     prn_ll_lines := print a linked list of lines
*/

#include "grade.h"

void append_to_score_line(FILE* ifp, FILE* ofp)
{
   char         line[maxline];
   char         name_line[maxline];
   char**       w;
   int          i;
   int          il_cnt;                             /* info line count */
   int          m;        /* number of the info line to be appended to */
   int          n;              /* number of data items to be appended */
   INFO_LINE*   hp;
   INFO_LINE*   p;

   hp = cll_info_lines(ifp);
   il_cnt = cnt_ll_lines(hp);
   if (il_cnt == 0) {
      printf("There are no info lines in your database - BYE.\n\n");
      return;
   }
   printf("Here is a snapshot of your database.\n\n");
   snapshot(ifp);
   printf("Here are the info lines extracted from the snapshot.\n\n");
   for (i = 1, p = hp; p != NULL; ++i, p = p -> next)
      printf("%3d%s", i, p -> line);
   printf("\nWhich line is to have data appended?  ");
   m = get_int_reply(1, il_cnt);              /* m is the magic number */
   /***=
     *   m  is the magic number.
     *   Every info line with count  m  has to be remade.
   =***/
   printf("\nHow many data items are to be appended to the line?  ");
   n = get_int_reply(1, maxline / 5);             /* rough upper bound */
   /***=
     *   We now set up work space that will be passed to remake().
     *   Since remake() is called repeatedly, it is inefficient
     *   to set up the work space within remake().
   =***/
   w = (char **) gcalloc((unsigned) n, sizeof(char *));
   for (i = 0; i < n; ++i)
      w[i] = (char *) gcalloc((unsigned) maxline, sizeof(char));
   /***=
     *   We are now ready to run through the input file
     *   remaking every info line with count  m .
   =***/
   printf("\nEnter your data.\n\n");
   while (fgets(line, maxline, ifp) != NULL && !isname(line))
      fputs(line, ofp);                /* run through the header lines */
   do {
      if (!isname(line)) {
         printf("%s\n%s\n\n",
            "SEVERE ERROR: the following is not a name line:",
            line);
         exit(1);
      }
      strcpy(name_line, line);
      fputs(name_line, ofp);                   /* output the name line */
      fputs(name_line, stdout);                 /* print the name line */
      for (i = 1; i <= il_cnt; ++i) {
         fgets(line, maxline, ifp);                       /* info line */
         if (i == m)                         /* magic info line number */
            remake(line, n, w, name_line);
         fputs(line, ofp);
      }
   } while (fgets(line, maxline, ifp) != NULL);    /* name line or EOF */
   printf("\n");
}
