#include "grade.h"

// ---
// cll_info_lines := create a linked list of info lines
// cll_nail := create a linked list of name and info lines
//
// The info lines between the first and second name line
// in the file pointed to by ifp are put into a linked list.
// The head pointer to the linked list is returned.
//
// Note: The input file is rewound at the end of cll_nail().
//

INFO_LINE *cll_info_lines(FILE* ifp)
{
   INFO_LINE*   hp;
   INFO_LINE*   p;

   hp = p = cll_nail(ifp);
   if (hp != NULL) {
      hp = hp -> next;
      free(p);
   }
   return hp;
}

// ---
// Rewind the input file and return the head pointer to the linked list.
//
// ---
// Mnemonics:
//
//    cll_nail := create a linked list of name and info lines
//

LINE* cll_nail(FILE* ifp)
{
   char   line[maxline], *s;
   LINE   *hp, *p;

   while ((s = fgets(line, maxline, ifp)) != NULL && !isname(line))
      ;                                /* run through the header lines */
   if (s == NULL) {
      rewind(ifp);
      return (NULL);
   }
   else {
      hp = p = (LINE *) malloc(sizeof(LINE));
      strcpy(p -> line, line);              /* first name line in file */
      while (fgets(line, maxline, ifp) != NULL && !isname(line)) {
	 p = p -> next = (LINE *) malloc(sizeof(LINE));
	 strcpy(p -> line, line);
      }
      p -> next = NULL;
      rewind(ifp);
      return (hp);
   }
}

// ---
// The score lines between the first and second name line in
// the file pointed to by ifp are put into a linked list.
// The head pointer to the linked list is returned.
//
// ---
// Remember:
//
//    The input file is rewound at the end of cll_nail().
//
// ---
// Mnemonics:
//
//    cll_score_lines := create a linked list of score lines
//    cll_nail := create a linked list of name and info lines
//
//

SCORE_LINE* cll_score_lines(FILE *ifp)
{
   INFO_LINE   *hp, *p, *q, *r;

   p = q = cll_nail(ifp);
   while (q != NULL && !isscore(q -> line)) {
      q = q -> next;
      free(p);
      p = q;
   }
   hp = r = q = p;                         /* first score line or NULL */
   while (p != NULL) {
      q = r = r -> next;
      while (r != NULL && !isscore(r -> line)) {
	 r = r -> next;
	 free(q);
	 q = r;
      }
      p = p -> next = r;
   }
   return hp;
}
