#include "grade.h"

char* get_abstract_info(char* answer, cchr* database)
{
   char        c, *p;
   int         i, k, cnt;
   FILE        *ifp;
   INFO_LINE   *hp, *ip;

   ifp = gfopen(database, "r");
   hp = cll_info_lines(ifp);
   if (hp == NULL) {
      printf("There are no info lines in your database.\n\n");
      gfclose(ifp);
      rll_lines(hp);
      *answer = '\0';
      return (answer);
   }
   printf("Here is a snapshot of your database.\n\n");
   snapshot(ifp);
   gfclose(ifp);
   printf("Here are the info lines extracted from the snapshot.\n\n");
   for (i = 1, ip = hp; ip != NULL; ++i, ip = ip -> next)
      printf("%3d%s", i, ip -> line);
   cnt = cnt_ll_lines(hp);
   rll_lines(hp);
   if (cnt == 1) {
      printf("\n%s%s",
	 "Do you want to use this line to ",
	 "print an abstract of the database?  ");
      c = reply();
      putchar('\n');
      if (c == 'y' || c == '\n')
	 strcpy(answer, "1");
      else
	 *answer = '\0';
      return (answer);
   }
   /***=
     *   From here on there is more than one choice.
   =***/
   printf("\n%s\n%s",
      "Each line can be used to print an abstract of the database.",
      "Which lines are to be used?  ");
   fgets(answer, maxline, stdin);
   putchar('\n');
   strip(answer);           /* remove beginning and ending white space */
   for (p = answer; isdigit(*p) || isspace(*p); ++p)
      ;
   if (*p != '\0') {
      printf("ERROR: Only integers can be input.\n\n");
      return (get_abstract_info(answer, database));
   }
   /***=
     *   From here on answer contains only digits and white space.
   =***/
   for (p = answer; *p != '\0'; ++p) {
      for ( ; isspace(*p); ++p)
	 ;
      sscanf(p, "%d", &k);
      if (k < 1 || k > cnt) {
	 printf("ERROR: You must nput integers between 1 and %d.", cnt);
	 return (get_abstract_info(answer, database));
      }
      for ( ; isdigit(*p); ++p)
	 ;
   }
   return (answer);
}

// ---
// get_fsfl := get first score field length
//

int get_fsfl(char* line)
{
   char    c;
   char*   p;
   char*   q;
   char*   r;

   r = kill_comment(line, &c);
   p = strchr(line, ':');
   if (p != NULL) {
      for (++p; isspace(*p); ++p)
	 ;
      if (*p != '\0') {
	 for (q = p + 1; !isspace(*q) && *q != '\0'; ++q)
	    ;
	 restore_comment(r, c);
	 return (q - p);
      }
   }
   restore_comment(r, c);
   return (0);
}

// ---
// The go[] array serves two important functions:
//    it marks which score lines have been selected;
//    it contains the number of scores on selected score lines.
//
// ---
// Note carefully:
//
//    the size of go[] matches the number of name and info lines
//    the size of go[] is nn := n + 1 where n = xxx
//    think of go[0] as corresponding to the name line
//
// ---
// Mnemonics:
//
//    a[] := array of selected info line numbers
//    m   := size of a[]
//    m   := number of info lines needed in the computation
//    n   := number of info lines
//    n   := # of info lines pointed to by hp
//    nn  := n + 1
//    nn  := size of go[]
//    hp  := head pointer to the linked list of info lines
//
// ---
// Note carefully:
//
//    the subscripts of a[] run from 0 to m - 1
//    the values of a[] are between 1 and n
//    a[i] <= a[i+1].
//
// ---
// Should this be an assertion?
//
//    true:  m <= n
//

void get_go_array(int* a, int m, int* go, int nn,
		  int* error_ptr, INFO_LINE* hp, int n)
{
   int         error, i, j;
   char        c, *q;
   INFO_LINE   *p;

   assert(m <= n);
   assert(nn == n + 1);
   go[0] = 0;
   j = 0;
   for (i = 1, p = hp; p != NULL; ++i, p = p -> next) {
      if (a[j] == i) {
	 go[i] = 1;
	 ++j;                                 /* index j only on a hit */
      }
      else
	 go[i] = 0;
   }
   /***=
     *   At this point go[i] = 1 if the ith info line has been
     *   selected by the user as a go line needed for the computation.
     *   Otherwise go[i] = 0.
     *
     *   Next, if the ith info line is a go line, we check to see
     *   that it is a score line, and if it is, we assign to go[i]
     *   the number of scores on that line.  If the ith info line
     *   is not a score line, error is set.
   =***/
   error = 0;
   for (i = 1, p = hp; p != NULL; ++i, p = p -> next) {
      if (go[i])
	 if (isscore(p -> line)) {
	    q = kill_comment(p -> line, &c);
	    go[i] = word_cnt(strchr(p -> line, ':') + 1);
	    restore_comment(q, c);
	 }
	 else {
	    printf("\n%s\n%s\n",
	       "ERROR: The following line is not a score line.", p -> line);
	    error = 1;
	    break;
	 }
   }
   *error_ptr = error;
}

// ---
// Get the .he line from the header file.
//
// A typical header file name is "class_header."
// The first line in this file must be the .he line.
// It is the same for both ditroff and nroff.
//
// If the header file does not exist, create it.
// If the .he line is missing, get it from the user
// and rewrite the header file.
//
// Ideally, the .he line should be created from the
// information contained in the file <database>_header.
// However, there is no convenient way to get all the
// elements needed.
//
// he_line = .he line             example: .he '...'...'...'
// hfn = header filename
//

static void   rewrt_file(char* he_line, char* hfn);
static void   wrt_lines_to_header_file(FILE* ofp);

void get_he_line(char* he_line, char* hfn)
{
   int     c;
   char    command[maxstring];
   char*   p;
   FILE*   ifp;
   FILE*   ofp;

   if ((ifp = fopen(hfn, "r")) != NULL) {
      if (fgets(he_line, maxline, ifp) == NULL) {
	 printf("\nSEVERE ERROR: No lines in the file %s  -  bye!\n\n", hfn);
	 exit(1);
      }
      gfclose(ifp);
      if (!is_nroff_he_line(he_line)) {
	 printf("\n%s\n\n%s\n\n%s\n\n%s\n\n%s",
	    "ERROR: An nroff header line of the form",
	    "   .he '...'...'...'",
	    "is needed.  Here is an example",
	    "   .he 'M143: COMPUTATIONAL MATH'Winter 1988'- % -'",
	    "Input your nroff header line:  ");
	 while (isspace(c = getchar()))
	    ;                              /* skip white space, if any */
	 ungetc(c, stdin);
	 fgets(he_line, maxline, stdin);
	 if (isspace(*he_line)) {
	    for (p = he_line; isspace(*p); ++p)
	       ;
	    strcpy(he_line, p);
	 }
	 if (is_nroff_he_line(he_line))
	    rewrt_file(he_line, hfn);
	 else
	    get_he_line(he_line, hfn);
      }
   }
   else {                                /* header file does not exist */
      printf("\n%s%s%s\n%s\n%s\n\n%s\n\n%s",
	 "FILE NEEDED: The file ", hfn, " will be created.",
	 "You will be put in vi to edit the file.",
	 "The top line must be of the form",
	 "   .he '...'...'...'",          /* an example is in the file */
	 "Ready?  ");
      if ((c = reply()) == 'y' || c == '\n') {
	 printf("\nStandby, vi is coming up ...\n\n");
	 ofp = gfopen(hfn, "w");
	 wrt_lines_to_header_file(ofp);
	 gfclose(ofp);
	 sprintf(command, "vi %s", hfn);
	 system(command);
      }
      get_he_line(he_line, hfn);
   }
}

// ---
// he_line = .he line             example: .he '...'...'...'
// hfn = header filename
//

static void rewrt_file(char* he_line, char* hfn)
{
   char    tfn[maxstring];                             /* tmp filename */
   int     c;
   FILE*   ifp;
   FILE*   ofp;

   tmpnam(tfn);
   ifp = gfopen(hfn, "r");
   ofp = gfopen(tfn, "w");
   fputs(he_line, ofp);
   while ((c = getc(ifp)) != EOF)
      putc(c, ofp);
   gfclose(ifp);
   gfclose(ofp);
   copy(tfn, hfn);
   gremove(tfn);
}

static void wrt_lines_to_header_file(FILE* ofp)
{
   fprintf(ofp, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
      ".he 'M143: COMPUTATIONAL MATH'Winter 1988'- % -'\n",
      "***\n",
      "***   Course:           M143: COMPUTATIONAL MATHEMATICS\n",
      "***\n",
      "***   Instructor:       Al Kelley\n",
      "***\n",
      "***   Time and place:   Tu-Th 8:00-9:45 in 175 Stevenson\n",
      "***\n",
      "***   TA:               Andy Duncan\n",
      "***\n",
      "***   Assistant:        Harry Gaver\n",
      "***\n",
      "***   Text:             A Book on C\n",
      "***                     by Al Kelley and Ira Pohl\n",
      "\n",
      "\n");
   fprintf(ofp, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
      "###   The perfect person would have the following scores:\n",
      "###\n",
      "###   PERFECT, PERSON\n",
      "###                          hmwk:     20  20  20  20  20  20  20  20\n",
      "###                          quiz:     20  20  20  20  20  20  20\n",
      "###                  section quiz:     20  20  20  20  20\n",
      "###                       midterm:    100\n",
      "###                         final:    100\n",
      "###          ((cut 3)) hmwk total:  100.0\n",
      "###          ((cut 5)) quiz total:  140.0\n",
      "###                wtd quiz total:  100.0\n",
      "###        wtd quiz-midterm total:  100.0\n",
      "###               wtd class total: 1000.0\n",
      "###                         grade:    A++\n",
      "\n",
      "\n");
   fprintf(ofp, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s",
      "###   HOW TOTALS ARE COMPUTED:\n",
      "###\n",
      "###      c3ht  := ((cut 3)) homework total\n",
      "###            := total of all but the three lowest homework scores\n",
      "###      c5qt  := ((cut 5)) quiz total\n",
      "###            := total of all but the five lowest homework scores\n",
      "###        wqt := weighted quiz total\n",
      "###            := (100 / 140) * c5qt\n",
      "###       wqmt := weighted quiz-midterm total\n",
      "###            := 0.75 * max(wqt, midterm) + 0.25 * min(wqt, midterm)\n",
      "###        wct := weighted class total\n",
      "###            := 2 * c3ht  +  4 * wqmt  +  4 * final\n",
      "###\n",
      "###   Thus the weighted class total is comprised of:\n",
      "###      20% homework\n",
      "###      40% quizzes and midterm\n",
      "###      40% final\n",
      "\n",
      "###   TO PASS THIS COURSE YOU MUST HAVE\n",
      "###   ---------------------------------\n",
      "###\n",
      "###      final >= 51     and     wtd class total >= 605\n",
      "###                      ---\n",
      "###\n");
}

// ---
// Get an integer from stdin and store it in n.
// The value of n must satisfy
//
//    n >= min     &&     n <= max
//

int get_int_reply(int min, int max)
{
   char    line[maxline];
   char*   p = line;
   int     n;

   fgets(line, maxline, stdin);
   strip(line);
   if (*line == '\0') {
      printf("ERROR: You must input an integer: ");
      return (get_int_reply(min, max));
   }
   if (*p == '+' || *p == '-')
      ++p;
   while (isdigit(*p))
      ++p;
   if (*p != '\0') {
      printf("ERROR: You must input an integer: ");
      return (get_int_reply(min, max));
   }
   sscanf(line, "%d", &n);
   if (n < min || n > max) {
      printf("%s\n%s%d%s%d%s",
	 "ERROR: Integer out of range.",
	 "You must input an integer between ", min, " and ", max, ":  ");
      return (get_int_reply(min, max));
   }
   return n;
}

// ---
// The user has indicated which line numbers are needed
// for the computation of a total score.
// The user's reply is in the string answer.
// These line numbers must be unique and must
// be between 1 and max_line_number.
//
// Check for errors and return the following:
//    the ordered array of line numbers a[],
//    its size m,
//    error.
//
// These values are passed back to the calling environment
// via a_ptr, m_ptr, and error_ptr.
//

void get_line_numbers(int* a_ptr[], int* m_ptr,
		      int* error_ptr, char* answer, int max_line_number)
{
   char*   p;
   int*    a;
   int     error;
   int     i;
   int     j;
   int     m;
   int     temp;

   for (p = answer; *p != '\0'; ++p)
      if (!isspace(*p) && !isdigit(*p)) {
	 *error_ptr = 1;
	 return;
      }
   m = word_cnt(answer);
   a = (int *) gcalloc((unsigned) m, sizeof(int));
   strip(answer);
   p = answer;
   for (i = 0; i < m; ++i) {
      sscanf(p, "%d", &a[i]);
      SET(p);
   }
   /***=
     *   Order the array.
   =***/
   for (i = 0; i < m - 1; ++i)
      for (j = i + 1; j < m; ++j)
	 if (a[i] > a[j]) {
	    temp = a[i];
	    a[i] = a[j];
	    a[j] = temp;
	 }
   error = 0;
   for (i = 0; !error && i < m - 1; ++i)
      error = a[i] == a[i+1];
   error = error || a[0] < 1 || a[m-1] > max_line_number;
   *a_ptr = a;
   *m_ptr = m;
   *error_ptr = error;
}

// ---
// Get a new phrase from the user to be used at the
// beginning of each new nonscore line.  Remember,
// since a tag is followed by a colon, a nonscore
// line cannot contain a colon.  Even though a
// beginning phrase serves the purpose of a
// tag, it cannot be called a tag.
//

void get_new_phrase(char* phrase, FILE* ifp)
{
   char   c, *p;

   printf("Here is a snapshot of your database.\n\n");
   snapshot(ifp);
   printf("%s\n\n%s\n\n%s\n\n%s\n\n%s\n\n%s",
      "Here is an example of a nonscore line:",
      "   project - simplex program in c",
      "If every corresponding nonscore line is of the form",
      "   project - .....",
      "then \"project - \" is the beginning phrase.",
      "Do you want to specify a beginning phrase?  ");
   c = reply();
   putchar('\n');
   if (c == 'n') {
      *phrase = '\0';
      return;                            /* no beginning phrase wanted */
   }
   *phrase = '\0';
   while (*phrase == '\0') {
      printf("Enter your desired beginning phrase: ");
      fgets(phrase, maxstring, stdin);
      for (p = phrase; isspace(*p); ++p)
	 ;
      if (p > phrase)                  /* delete beginning white space */
	 strcpy(phrase, p);
      for (p = phrase; *p != '\n'; ++p)
	 ;
      *p = '\0';
      for (p = phrase; isspace(*p); ++p)
	 ;
      if (*p == '\0')                   /* all white space not allowed */
	 *phrase = '\0';
      else {
	 printf("%s\n\n%s%s%s\n\n%s",
	    "Here is your beginning phrase:",
	    "   \"", phrase, "\"",
	    "Is it correct?  ");
	 if (reply() == 'n')
	    *phrase = '\0';
	 putchar('\n');
      }
   }
}

// ---
// Get a new tag from the user.
//

void get_new_tag(char* tag, FILE* ifp)
{
   char*   p;
   char    temp[maxline];

   printf("Here is a snapshot of your database.\n\n");
   snapshot(ifp);
   printf("Enter your new tag: ");
   get_string_reply(tag);
   printf("\n");
   /*
   // Make sure that tag ends with a colon.
   */
   if (*((p = strchr(tag, '\0')) - 1) != ':') {
      *p = ':';
      *(p + 1) = '\0';
   }
   strcpy(temp, tag);
   strcpy(tag, blank_spaces(mnbsbt));
   strcat(tag, temp);
   /*
   // The variable tag now begins with blank spaces and ends with a colon.
   */
   assert(isspace(*tag));
   assert(*(tag + strlen(tag) - 1) == ':');
}

int get_option_from_list()
{
   char   answer[maxstring];
   char   pgm_name[maxword];
   int    option;

   assert(Quit == 10);
   strcpy(pgm_name, o.local_pgm_name());
   capitalize(pgm_name);
   //
   // The following list must be coordinated
   // with corresponding values in grade.h.
   //
   cout << "---\n"
        << pgm_name << ": List of options:\n"
           "\n"
           "   0  instructions\n"
           "   1  enter scores or lines\n"
           "   2  compute total scores\n"
           "   3  assign grades / write evaluations\n"
           "   4  compute statistics\n"
           "   5  pretty print\n"
           "   6  print on paper\n"
           "   7  publish for students\n"
           "   8  reorder / rework the database\n"
           "   9  view the database\n"
           "  " << Quit << "  quit\n"
	   "\n"
           "Enter your option: ";
   cout.flush();
   get_string_reply(answer);
   cout << endl;
   if (*answer == 'i')
      option = Instructions;
   else if (*answer == 'e')
      option = Enter_scores_or_lines;
   else if (strncmp(answer, "compute t", 9) == 0)
      option = Compute_total_scores;
   else if (strncmp(answer, "total", 5) == 0)
      option = Compute_total_scores;
   else if (*answer == 'a')
      option = Assign_grades;
   else if (strncmp(answer, "compute s", 9) == 0)
      option = Compute_statistics;
   else if (strncmp(answer, "pp", 2) == 0)
      option = Pretty_print;
   else if (strncmp(answer, "pre", 3) == 0)
      option = Pretty_print;
   else if (*answer == 'r')
      option = Reorder_the_database;
   else if (strncmp(answer, "pri", 3) == 0)
      option = Print_on_paper;
   else if (*answer == 'v')
      option = View;
   else if (*answer == 'q')
      option = Quit;
   else
      option = extract_int(answer, 0, Quit);
   return option;
}

// ---
// Skip the first score.
// Find the max field length of remaining scores in line.
//
// get_rsfl := get remaining scores field length
//

int get_rsfl(char* line)
{
   char    c;
   char*   p;
   char*   q;
   char*   r;
   int     d;
   int     rsfl = 0;

   r = kill_comment(line, &c);
   p = strchr(line, ':');
   if (p != NULL) {
      SET(p);                            /* first score or end-of-line */
      if (*p != '\0')
	 do {
	    SET(p);
	    for (q = p; !isspace(*q) && *q != '\0'; ++q)
	       ;
	    d = q - p;
	    rsfl = (d > rsfl) ? d : rsfl;
	 } while (*p != '\0');
      restore_comment(r, c);
      return rsfl;
   }
   restore_comment(r, c);
   return 0;
}

// ---
// Catch a nonwhite space answer.
//

char* get_string_reply(char* answer)
{
   char   line[maxline];

   *line = '\0';
   while (*line == '\0') {
      fgets(line, maxline, stdin);
      strip(line);
   }
   return (strcpy(answer, line));
}

// ---
// tfl := tag field length
//

int get_tfl(char* line)
{
   char*   p = line;
   char*   q;

   while (isspace(*p))
      ++p;
   q = strchr(p, ':');
   return (q - p + 1);
}

// ---
// The size of weight[] is the nscores.
//
// n    := number of info lines
// nn   := n + 1, to match subscripts with info line numbers
// nn   := size of go[]
// head := head pointer to linked list of info lines
//

dbl* get_weight_array(int* nscores_ptr, int go[],
		      int nn, INFO_LINE* head, int w_sw)
{
   int          conversion_cnt;
   int          error;
   int          i;
   int          j;
   int          k;
   int          n;
   int          nscores;
   char         c;
   char         discard[maxline];
   char         line[maxline];
   dbl*         weight;
   FILE*        wfp;                           /* weights file pointer */
   INFO_LINE*   p;

   n = cnt_ll_lines(head);
   assert(n + 1 == nn);
   nscores = 0;
   for (i = 1, p = head; p != NULL; ++i, p = p -> next)
      nscores += go[i];
   weight = (double *) gcalloc((unsigned) nscores, sizeof(double));
   if (w_sw == 0) {
      j = 0;
      for (i = 0; i < nscores; ++i)
	 weight[j++] = 1.0;
   }
   else {
      do {
	 do {
	    printf("%s\n%s\n\n",
	       "Weights are needed.",
	       "Here is a snapshot of your selected score lines:");
	    prn_ll_go_lines(go, nn, head);
	    printf("%s%d\n%s\n%s\n%s\n%s",
	       "Number of scores: ", nscores,
	       "Corresponding weights are needed.",
	       "Enter the name of a file that contains",
	       "the weights, or enter the weights directly:",
	       "   ");
	    while (isspace(c = getchar()))
	       ;
	    ungetc(c, stdin);
	    printf("\n");
	    error = 0;
	    if (isdigit(c) || c == '.')
	       wfp = stdin;
	    else {
	       fgets(line, maxline, stdin);
	       strip(line);
	       if ((wfp = fopen(line, "r")) == NULL) {
		  error = 1;
		  printf("Cannot open %s.\n\nDo it again.\n\n", line);
	       }
	    }
	 } while (error);
	 k = 0;
	 for (i = 1; !error && i < nn; ++i)
	    for (j = 0; !error && j < go[i]; ++j) {
	       conversion_cnt = fscanf(wfp, "%lf", &weight[k++]);
	       error = conversion_cnt == 0 || conversion_cnt == EOF;
	    }
	 if (wfp == stdin)
	    fgets(discard, maxline, stdin);
	 else
	    gfclose(wfp);
	 if (error)
	    printf("ERROR: Do it again.\n\n");
      } while (error);
   }
   *nscores_ptr = nscores;
   return weight;
}
