// ---
// This function formats score lines so that:
//    each tag begins with at least mnbsbt blank spaces,
//    all tags are lined up on  : ,
//    each score begins with at least mnbsbs blank spaces,
//    all scores are aligned.
//
// mnbsbt := minimum number of blank spaces before tags
// mnbsbs := minimum number of blank spaces before scores
//
// The extract...() functions are used to find desired strings,
// to format the strings appropriately, and to put the strings
// into s1, s2, s3.
//

#include "grade.h"

void pretty_print(cchr* database)
{
   char    line[maxline];
   char    s1[maxline];
   char    s2[maxline];
   char    s3[maxline];
   char    tfn[maxstring];

   int     tfl;                                    /* tag field length */
   int     fsfl;                           /* first score field length */
   int     rsfl;                      /* remaining scores field length */

   int     max_tfl = 0;
   int     max_fsfl = 0;
   int     max_rsfl = 0;
   FILE*   ifp;
   FILE*   ofp;

   /*
   // First run through the file pointed to by ifp
   // to find max_tfl, max_fsfl, and max_rsfl.
   */
   tmpnam(tfn);
   startup(database, &ifp, tfn, &ofp);
   printf("Pretty printing the database ...\n\n");
   while (fgets(line, maxline, ifp) != NULL && !isname(line))
      ;                                           /* skip header lines */
   while (fgets(line, maxline, ifp) != NULL)
      if (isscore(line)) {
         tfl = get_tfl(line);
         max_tfl = (tfl > max_tfl) ? tfl : max_tfl;
         fsfl = get_fsfl(line);
         max_fsfl = (fsfl > max_fsfl) ? fsfl : max_fsfl;
         rsfl = get_rsfl(line);
         max_rsfl = (rsfl > max_rsfl) ? rsfl : max_rsfl;
         }
   rewind(ifp);
   /***
     *   Now do the pretty printing.
     *   Begin by writing out the header lines.
   ***/
   while (fgets(line, maxline, ifp) != NULL && !isname(line))
      fputs(line, ofp);
   if (isname(line))                      /* found the first name line */
      fputs(line, ofp);
   while (fgets(line, maxline, ifp) != NULL) {
      if (isscore(line)) {
         extract_tag(line, max_tfl, s1);
         extract_scores(line, max_fsfl, max_rsfl, s2);
         extract_comment(line, s3);
         strcpy(line, s1);                          /* remake the line */
         strcat(line, s2);
         strcat(line, s3);
         strcat(line, "\n");
      }
      fputs(line, ofp);
   }
   wrapup(database, ifp, tfn, ofp);
}
