#include "grade.h"
#include "print_on_paper.h"                // used in wrt_ditroff_lines()

// ---
// Write ditroff command lines.
//
// The .he line is the same for ditroff and nroff.
//

#define   FNAME   "class_ditroff_header"

void wrt_ditroff_lines(cchr* database, FILE* ofp)
{
   char   hfn[maxstring];                          /* header file name */
   char   he_line[maxline];                      /* .he  '...'...'...' */

   sprintf(hfn, "%s_header", database);
   get_he_line(he_line, hfn);
   fprintf(ofp, "%s%s%.1f%s%s%.1f%s%s%s%s%s%s%s%s%s%s%s",
      ".in 0\n",
      ".ll ", LL, "c\n",
      ".po ", PO, "c\n",
      ".ps 8\n",
      ".vs 10p\n",
      ".m1 2v   \\\"space between the top of the page to the header\n",
      ".m2 3v   \\\"space between the header and the first line\n",
      ".m3 2v   \\\"space between the bottom of the text and the footer\n",
      ".m4 2v   \\\"space between the footer and the bottom of the page\n",
      he_line,
      ".fo '###'\\n(dy \\*(mo 19\\n(yr'###'\n",
      ".nf\n",
      ".bp 1\n");
}

// ---
// Wednesday   12:10:41 PST   25 February 1987
//   
// Implement a simple template scheme to write evaluations.
//   
// For each course the registrar provides a file that contains
// the names of all the students.  Written evaluations for each
// student can be entered into the file.
//   
// This function assumes that grades have been assigned for each
// student.  For example:
//   
//    NAME: EDELSON, DANIEL ROSS   ID: 566556382  F86-COURSE: 40900  ...
//    a+ a+ c a+
//    NAME: GAVER, HARRY LAWRENCE  ID: 175369005  F86-COURSE: 40900  ...
//    a a b+ a
//    NAME: GREEN, DANIEL RUSSELL  ID: 564041798  F86-COURSE: 40900  ...
//    a a a a
//   
// The file of names from the registrar, with grades appropriately
// inserted, becomes the database for the grade program.  Suppose,
// for example, that the name of the database is 40900, the number
// of the course used by the registrar.  A corresponding file
//   
//    40900_template
//   
// is used to translate grades to words.  For example, the following
// lines might be in 40900_template:
//   
//    a+ = Outstanding
//    a = Excellent
//    b+ = Very good
//    c = Adequate
//    f = Failing
//    i = Incomplete
//    $ = No
//    $ = No
//    $ = No
//    $ = Failing
//    :::::::::::::::::::::::::::::::::::::::::::::::::::::::
//    $ quiz scores.  $ class project.  $ final examination.
//    $ overall performance.
//

void wrt_evaluations(cchr* database, int test_mode)
{
   cchr*   dummy1 = database;                  // keep the compiler happy
   int     dummy2 = test_mode;                 // keep the compiler happy

   // ---
   // Keep the g++ compiler happy.
   //
   ++dummy1;
   ++dummy2;
   cout << "SORRY: Not yet implemented.\n\n";
}

#if 0
   char    line[maxline];
   char    tfn[maxstring];
   int     nrepl;                               // number of replacements
   FILE*   ifp;
   FILE*   ofp;

   tmpnam(tfn);
   startup(database, &ifp, tfn, &ofp);
   nrepl = chk_for_wrt_eval_errors(database);
   trans = translation_array(nrepl);
   repl = replacement_lines(...);
   while (fgets(line, maxline, ifp) != NULL && !isname(line))
      //
      // Run through the header lines.
      //
      fputs(line, ofp);
   fputs(line, ofp);                               // the first name line
   while (fgets(line, maxline, ifp) != NULL) {
      if (isname(line))
         fputs(line, ofp);
      else {                                       // write an evaluation
         
   wrapup(database, ifp, tfn, ofp);
}
#endif

void wrt_help(cchr* pgm_name)
{
   cchr*&   pg = pgm_name;

   cout << "---\n"
           "Usage:     " << pg << "  [-h[h[h]]]  [filename]\n"
           "\n"
           "---\n"
           "Examples:  " << pg << "  class\n"
           "           " << pg << "  -h\n"
           "           " << pg << "  -hh\n"
           "           " << pg << "  -hhh\n"
           "           " << pg << "  -t\n"
           "\n"
           "---\n"
           "Options:\n"
           "   -h     help option, print this message\n"
           "   -hh    help help option, print a more serious message\n"
           "   -hhh   help help help option, print a very long message\n"
           "   -t     test option, print information for the programmer\n"
           "\n";
}

//  ---
//  29 Jun 93
//
//  Peter Scott wants long messages to go through more.
//  As of today, this wish has been granted.
//

void wrt_more_help(cchr* pgm_name)
{
   char         tfn[maxstring];
   char*        cmnd;
   ofstream     ofs;
   ostrstream   oss;

   cchr*&       pg = pgm_name;

   tmpnam(tfn);
   ofs.open(tfn);
   assert(ofs);
   ofs <<
      "\n"
      "---\n"
      "To get started, do the following:\n"
      "\n"
      "   mkdir $home/phys7b    -- make a directory for your class\n"
      "   cd $home/phys7b       -- change directory to where class info is\n"
      "   vi class              -- make a list of student names in this file\n"
      "\n"
      "Before you make a serious list of names, play with a toy list.\n"
      "For example, put the following names in the file named class:\n"
      "\n"
      "   MOUSE, MICKEY\n"
      "   DUCK, DONALD\n"
      "   MAN, SUPER\n"
      "   WOMAN, WONDER\n"
      "\n"
      "Caution: The names must begin in column 1.  In this program, the file\n"
      "named \"class\" is called the database.  You are now ready to see how the\n"
      "program manipulates the database.  Give the command\n"
      "\n"
      "   " << pg << " class\n"
      "\n"
      "Try many of the options, viewing the database as you go along.\n"
      "If you run into serious trouble, send me email.\n"
      "\n"
      "---\n"
      "If you wish, you may use as the database the file of student names\n"
      "supplied by the registrar.  Here is an example of such a file:\n"
      "\n"
      "  Date: Thu, 19 Nov 92 17:45:37 -0800\n"
      "  From: script@ucsce.UCSC.EDU (SCRIPT Center, x4682)\n"
      "  To: blufox@cats.UCSC.EDU\n"
      "  Subject: COMPUTATION MATH I; KELLEY,A. (Call #91732-F92)\n"
      "\n"
      "  Sched. #: 91732 Course: COMPUTATION MATH I          Instr.: KELLEY,A.         \n"
      "  Enrolled: Thu Nov 19 17:04:26 1992              At Pool: 05\n"
      "  Machine: sasha                Account: blufox  \n"
      "\n"
      "  NAME: ALLEN, APRIL ELIZABET  ID: 565752503  F92-COURSE: 91732  ST: 05  SEX: F\n"
      "  NAME: ALLEN, NANCY TUFTS     ID: 564986471  F92-COURSE: 91732  ST: 05  SEX: F\n"
      "  NAME: BUNAS, KIRBY LYNNE     ID: 566394311  F92-COURSE: 91732  ST: 05  SEX: F\n"
      "  .........\n"
      "\n"
      "Suppose that you have copied a file supplied by the registrar into\n"
      "the file named class.  Now you are ready to give the command\n"
      "\n"
      "   " << pg << " class\n"
      "\n"
      "The scores that you enter on a score line can be any string of characters.\n"
      "For example,\n"
      "\n"
      "   ALLEN, April\n"
      "      hmwk:    89  late   100\n"
      "      comment - trying harder\n"
      "   ALLEN, Nancy\n"
      "      hmwk:   100    A+    B-\n"
      "      comment - seems to be slipping\n"
      "   BUNAS, Kirby\n"
      "      hmwk:  fair  good    77\n"
      "      comment - making a lot of improvement\n"
      "   \n"
      "Name lines contain names, score lines have a tag name followed by a colon\n"
      "followed by zero or more scores, and anything else is a nonscore line.\n"
      "The entries after each name have to match in kind, and the number\n"
      "of scores on the n_th score line after each name must be\n"
      "consistent.\n"
      "\n";
   //
   // Peter Scott has requested that
   // it all be piped through more.
   //
   ofs.close();
   oss << "more " << tfn << ends;
   cmnd = oss.str();
   gsystem(cmnd);
   delete [] cmnd;
   gremove(tfn);
}

void wrt_much_more_help(cchr* pgm_name)
{
   char         tfn[maxstring];
   char*        cmnd;
   ofstream     ofs;
   ostrstream   oss;

   cchr*&       pg = pgm_name;

   tmpnam(tfn);
   ofs.open(tfn);
   assert(ofs);
   ofs <<
      "---\n"
      "From: blufox@cats.ucsc.edu  (Al Kelley)\n"
      "To: Peter Scott <drip@cats.UCSC.EDU>\n"
      "Subject: Re: software for grading \n"
      "\n"
      "---\n"
      "If a T.A. is going to do all the data entry, then probably\n"
      "it is best for her to do it in her space and then mail you\n"
      "a copy after each update.  The data entry is the hardest\n"
      "part of the whole grading process. Here are the rules:\n"
      "\n"
      "   1  If you start entering scores, you have to continue to the end.\n"
      "      There is no way to stop in the middle and continue with the\n"
      "      chore later.\n"
      "   \n"
      "   2  Use the character $ to enter a void score.\n"
      "\n"
      "   3  If you enter a wrong score, there is no way to back up.\n"
      "      Since that feature has never been added to this package,\n"
      "      you must write down any corrections as you go along.\n"
      "      Do not rely on memory or you will end up with an\n"
      "      incorrect database.\n"
      "\n"
      "   4  Corrections get entered with vi, and of course, vi can\n"
      "      be used to make global changes to your tags, or to change\n"
      "      the spelling of a name, or to enter a new name, etc.\n"
      "\n"
      "   5  Just so you will have an idea of what can be done at the\n"
      "      end of the quarter, I will attach a real example.\n"
      "\n"
      "---\n"
      "ALFORS, BARBARA\n"
      "                     hmwk:     19  19  18  20  14  20  19  18  20\n"
      "                     quiz:     10  10   9  10   8   7   6  10  10  10  10\n"
      "                   matlab:      9   $\n"
      "                  midterm:     97\n"
      "                    final:     81\n"
      "   (cut 3) wtd hmwk total:   97.5\n"
      "   (cut 3) wtd quiz total:   96.0\n"
      "   wtd quiz-midterm total:   96.8\n"
      "          wtd class total:  906.2\n"
      "                    grade:      A\n"
      "BE, MELINDA\n"
      "                     hmwk:     20  20  20  20  19  20  15  20   $\n"
      "                     quiz:    10L   9   5   6   4   4   2   4   4  10  10\n"
      "                   matlab:     10  10\n"
      "                  midterm:     70\n"
      "                    final:     69\n"
      "   (cut 3) wtd hmwk total:  100.0\n"
      "   (cut 3) wtd quiz total:   78.0\n"
      "   wtd quiz-midterm total:   76.0\n"
      "          wtd class total:  780.0\n"
      "                    grade:      B\n"
      ".........\n"
      "\n"
      "---\n"
      "   6  All the lines containing the word \"total\" are computed\n"
      "      by the " << pg << " program, as well as the line containing\n"
      "      the word \"grade.\"  Those lines illustrate the real\n"
      "      value of the " << pg << " program.\n"
      "   \n"
      "   7  When I publish scores, I also publish a cover sheet.  This\n"
      "      cover sheet has to be made by hand.  Here is the cover\n"
      "      sheet that goes with the above database.\n"
      "\n"
      "---\n"
      ".he 'M12: LINEAR ALGEBRA'Spring 1991'- % -'\n"
      "\n"
      "//   Course:           M112: LINEAR ALGEBRA\n"
      "//\n"
      "//   Instructor:       Al Kelley\n"
      "//\n"
      "//   Time and place:   MWF 11:00 - 12:10 in 283 Kerr Hall\n"
      "//\n"
      "//   TA:               Bin Shao\n"
      "//   Reader:           Tuyen Nguyen\n"
      "//   Reader:           Josh Lubliner\n"
      "//   Resource:         Jim Howard\n"
      "//\n"
      "//   Text:             Elementary Linear Algebra\n"
      "//                     by Howard Anton\n"
      "//                     John Wiley & Sons, Inc., 1991\n"
      "\n"
      "###   The perfect person would have the following scores:\n"
      "###\n"
      "###   PERFECT, PERSON\n"
      "###                     hmwk:     20  20  20  20  20  20  20  20  20\n"
      "###                     quiz:     10  10  10  10  10  10  10  10  10  10  10\n"
      "###                   matlab:     10  10\n"
      "###                  midterm:    100\n"
      "###                    final:    100\n"
      "###   (cut 3) wtd hmwk total:  100.0\n"
      "###   (cut 3) wtd quiz total:  100.0\n"
      "###   wtd quiz-midterm total:  100.0\n"
      "###          wtd class total: 1000.0\n"
      "###                    grade: A++\n"
      "\n"
      "###   HOW TOTALS ARE COMPUTED:\n"
      "###\n"
      "###      c3ht  := (cut 3) homework total\n"
      "###            := total of all but the three lowest homework scores\n"
      "###      c3qt  := (cut 3) quiz total\n"
      "###            := total of all but the three lowest homework scores\n"
      "###        wht := (cut 3) weighted homework total\n"
      "###            := (100 / 120) * c3ht\n"
      "###        wqt := weighted quiz total\n"
      "###            := (100 / 100) * c3qt\n"
      "###       wqmt := weighted quiz-midterm total\n"
      "###            := 0.75 * max(wqt, midterm) + 0.25 * min(wqt, midterm)\n"
      "###        wct := weighted class total\n"
      "###            := 2 * wht  +  4 * wqmt  +  4 * final\n"
      "###\n"
      "###   Thus the weighted class total is comprised of:\n"
      "###      20% homework\n"
      "###      40% quizzes and midterm\n"
      "###      40% final\n"
      "\n"
      "###   TO PASS THIS COURSE YOU MUST HAVE\n"
      "###   ---------------------------------\n"
      "###\n"
      "###      final >= 60     and     wtd class total >= 695\n"
      "###                      ---\n"
      "\n"
      "---\n"
      "   8  As you can imagine, this cover sheet is painful to make.\n"
      "      I always copy an old one and make changes as necessary.\n"
      "   \n"
      "   9  Technical note:\n"
      "      ---\n"
      "      This is the only serious program that I have written that uses\n"
      "      fork().  Right at the beginning, I fork the program so that I\n"
      "      can do a check on the database and at the same time continue\n"
      "      with the main thrust of the work that is going to be done.\n"
      "      Since most of the time the database passes the check, most of\n"
      "      the time there is no problem.  If, however, the database does\n"
      "      not pass the check, then the two forks do not work together as\n"
      "      they should.  At the time I wrote the code, I did not know how\n"
      "      to make it all happen correctly.  I really should fix it, but,\n"
      "      well, .... .  To see what the confusion is, try the following\n"
      "      command:\n"
      "\n"
      "         " << pg << " xxx\n"
      "      \n"
      "      where xxx is a file that does not exist.  You will see one\n"
      "      fork saying \"Bye,\" while the other fork continues merrily on\n"
      "      its way -- not an optimal way to proceed.\n"
      "\n"
      "  10  In addition to fixing the fork() problem, are there any other\n"
      "      improvements that this software package needs, you might ask.\n"
      "      Well, as always there are things that need to be done.  One\n"
      "      improvement would be a decent interface to tex and postscript\n"
      "      so that the database can be readily printed.  At the moment,\n"
      "      printing is done via nroff and is tied to the Xerox laser\n"
      "      printer at the computer center.\n"
      "      \n"
      "  11  Finally, everything should be moved to X-windows.  However,\n"
      "      since I know very little about X-windows, that move will\n"
      "      be a lot of work.\n"
      "\n"
      "\n";
   //
   // Peter Scott has requested that
   // it all be piped through more.
   //
   ofs.close();
   oss << "more " << tfn << ends;
   cmnd = oss.str();
   gsystem(cmnd);
   delete [] cmnd;
   gremove(tfn);
}

// ---
// Write nroff command lines.
//
// A typical header file name is "class_header."
// The first line in this file is the .he line.
// It is the same for both ditroff and nroff.
//

void wrt_nroff_lines(cchr* database, FILE* ofp)
{
   char   hfn[maxstring];                          /* header file name */
   char   he_line[maxline];                      /* .he  '...'...'...' */

   sprintf(hfn, "%s_header", database);
   get_he_line(he_line, hfn);
   fprintf(ofp, "%s%s%s%s%s%s%s%s%s%s%s%s",
      "\\!$$DJDE FORMAT=12CPI DUPLEX=NO DATA=(0,102) END;\n",
      ".in 0\n",
      ".ll 20.7c   \\\"old value  =  .ll 17c    *** 12CPI, not 10CPI ***\n",
      /* coordinate .ll with print roster routine */
      ".po 3c      \\\"old value  =  .po 2.3c   *** 12CPI, not 10CPI ***\n",
      ".m1 2v      \\\"space between the top of the page to the header\n",
      ".m2 3v      \\\"space between the header and the first line\n",
      ".m3 2v      \\\"space between the bottom of the text and the footer\n",
      ".m4 2v      \\\"space between the footer and the bottom of the page\n",
      he_line,
      ".fo '###'\\n(dy \\*(mo 19\\n(yr'###'\n",
      ".nf\n",
      ".bp 1\n");
}
